/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "lua_func_crc.h"

#include "lauxlib.h"

#include <stdint.h>

#include "lua_helper.h"
#include "crc.h"

//-----------------------------------------------------------------------------

uint8_t   g_crc8_polynom = CRC8_POLYNOM_1_WIRE;
uint16_t  g_crc16_polynom = CRC16_POLYNOM_IBM;
uint32_t  g_crc32_polynom = CRC32_POLYNOM_IEEE;

//-----------------------------------------------------------------------------

int lua_func_crc8(lua_State *lua)
{
  uint8_t crc = (uint8_t)luaL_checknumber(lua, 1);
  
  if (lua_isnumber(lua, 2))
  {
    uint8_t data = (uint8_t)luaL_checknumber(lua, 2);
    
    crc8_calc(&crc, g_crc8_polynom, data);
  }
  else if (lua_istable(lua, 2))
  {
    for (lua_pushnil(lua); lua_next(lua, 2); lua_pop(lua, 1))
    {
      uint8_t data = (uint8_t)luaL_checknumber(lua, -1);
      
      crc8_calc(&crc, g_crc8_polynom, data);
    }
  }
  else
  {
    lua_throw_error_from_c(lua, "parameter 2 must be a number or an array");
  }
  
  lua_pushnumber(lua, (lua_Number)crc);
  return 1;
}

//-----------------------------------------------------------------------------

int lua_prop_crc8_polynom_read(lua_State *lua)
{
  lua_pushnumber(lua, (lua_Number)g_crc8_polynom);
  return 1;
}

//-----------------------------------------------------------------------------

int lua_prop_crc8_polynom_write(lua_State *lua)
{
  g_crc8_polynom = (uint8_t)luaL_checknumber(lua, 3);
  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_crc16(lua_State *lua)
{
  uint16_t crc = (uint16_t)luaL_checknumber(lua, 1);
  
  if (lua_isnumber(lua, 2))
  {
    uint8_t data = (uint8_t)luaL_checknumber(lua, 2);
    
    crc16_calc(&crc, g_crc16_polynom, data);
  }
  else if (lua_istable(lua, 2))
  {
    for (lua_pushnil(lua); lua_next(lua, 2); lua_pop(lua, 1))
    {
      uint8_t data = (uint8_t)luaL_checknumber(lua, -1);
      
      crc16_calc(&crc, g_crc16_polynom, data);
    }
  }
  else
  {
    lua_throw_error_from_c(lua, "parameter 2 must be a number or an array");
  }
  
  lua_pushnumber(lua, (lua_Number)crc);
  return 1;
}

//-----------------------------------------------------------------------------

int lua_prop_crc16_polynom_read(lua_State *lua)
{
  lua_pushnumber(lua, (lua_Number)g_crc16_polynom);
  return 1;
}

//-----------------------------------------------------------------------------

int lua_prop_crc16_polynom_write(lua_State *lua)
{
  g_crc16_polynom = (uint16_t)luaL_checknumber(lua, 3);
  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_crc32(lua_State *lua)
{
  uint32_t crc = (uint32_t)luaL_checknumber(lua, 1);
  
  if (lua_isnumber(lua, 2))
  {
    uint8_t data = (uint8_t)luaL_checknumber(lua, 2);
    
    crc32_calc(&crc, g_crc32_polynom, data);
  }
  else if (lua_istable(lua, 2))
  {
    for (lua_pushnil(lua); lua_next(lua, 2); lua_pop(lua, 1))
    {
      uint8_t data = (uint8_t)luaL_checknumber(lua, -1);
      
      crc32_calc(&crc, g_crc32_polynom, data);
    }
  }
  else
  {
    lua_throw_error_from_c(lua, "parameter 2 must be a number or an array");
  }
  
  lua_pushnumber(lua, (lua_Number)crc);
  return 1;
}

//-----------------------------------------------------------------------------

int lua_prop_crc32_polynom_read(lua_State *lua)
{
  lua_pushnumber(lua, (lua_Number)g_crc32_polynom);
  return 1;
}

//-----------------------------------------------------------------------------

int lua_prop_crc32_polynom_write(lua_State *lua)
{
  g_crc32_polynom = (uint32_t)luaL_checknumber(lua, 3);
  return 0;
}

//-----------------------------------------------------------------------------

