
/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "lua_func_gfx.h"

#include "lauxlib.h"
#include "lstate.h"

#include "InterfaceGUI.h"

//-----------------------------------------------------------------------------

int lua_func_gfx_clear(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  iGui->GfxClear();

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_draw_pnt(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);

  iGui->GfxDrawPoint(x, y);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_draw_line(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x1 = luaL_checknumber(lua, 1);
  double  y1 = luaL_checknumber(lua, 2);
  double  x2 = luaL_checknumber(lua, 3);
  double  y2 = luaL_checknumber(lua, 4);

  iGui->GfxDrawLine(x1, y1, x2, y2);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_draw_circle(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);
  double  r = luaL_checknumber(lua, 3);

  iGui->GfxDrawCircle(x, y, r);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_draw_ellipse(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);
  double  rx = luaL_checknumber(lua, 3);
  double  ry = luaL_checknumber(lua, 4);

  iGui->GfxDrawEllipse(x, y, rx, ry);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_draw_rect(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);
  double  w = luaL_checknumber(lua, 3);
  double  h = luaL_checknumber(lua, 4);

  iGui->GfxDrawRect(x, y, w, h);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_draw_polygon(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  QPolygon polygon;

  luaL_checktype(lua, 1, LUA_TTABLE);

  for (lua_pushnil(lua); lua_next(lua, 1); lua_pop(lua, 1))
  {
    luaL_checktype(lua, -1, LUA_TTABLE);

    lua_rawgeti(lua, -1, 1);
    double x = luaL_checknumber(lua, -1);
    lua_pop(lua, 1);

    lua_rawgeti(lua, -1, 2);
    double y = luaL_checknumber(lua, -1);
    lua_pop(lua, 1);

    polygon << QPoint(x, y);
  }

  iGui->GfxDrawPolygon(polygon);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_fill_circle(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);
  double  r = luaL_checknumber(lua, 3);

  iGui->GfxFillCircle(x, y, r);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_fill_ellipse(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);
  double  rx = luaL_checknumber(lua, 3);
  double  ry = luaL_checknumber(lua, 4);

  iGui->GfxFillEllipse(x, y, rx, ry);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_fill_rect(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);
  double  w = luaL_checknumber(lua, 3);
  double  h = luaL_checknumber(lua, 4);

  iGui->GfxFillRect(x, y, w, h);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_fill_polygon(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  QPolygon polygon;

  luaL_checktype(lua, 1, LUA_TTABLE);

  for (lua_pushnil(lua); lua_next(lua, 1); lua_pop(lua, 1))
  {
    luaL_checktype(lua, -1, LUA_TTABLE);

    lua_rawgeti(lua, -1, 1);
    double x = luaL_checknumber(lua, -1);
    lua_pop(lua, 1);

    lua_rawgeti(lua, -1, 2);
    double y = luaL_checknumber(lua, -1);
    lua_pop(lua, 1);

    polygon << QPoint(x, y);
  }

  iGui->GfxFillPolygon(polygon);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_draw_text(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);

  iGui->GfxDrawText(x, y, luaL_checkstring(lua, 3));

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_set_color(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  if (lua_isnumber(lua, 1))
  {
    int red = luaL_checkinteger(lua, 1);
    int green = luaL_checkinteger(lua, 2);
    int blue = luaL_checkinteger(lua, 3);

    iGui->GfxSetColor(red, green, blue);
  }
  else
  {
    iGui->GfxSetColor(luaL_checkstring(lua, 1));
  }

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_set_fill_color(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  if (lua_isnumber(lua, 1))
  {
    int red = luaL_checkinteger(lua, 1);
    int green = luaL_checkinteger(lua, 2);
    int blue = luaL_checkinteger(lua, 3);

    iGui->GfxSetFillColor(red, green, blue);
  }
  else
  {
    iGui->GfxSetFillColor(luaL_checkstring(lua, 1));
  }

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_begin_update(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;
  iGui->GfxBeginUpdate();
  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_end_update(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;
  iGui->GfxEndUpdate();
  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_set_size(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);

  iGui->GfxSetSize(x, y);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_gfx_save_img(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  QString fileName = luaL_checkstring(lua, 1);

  iGui->GfxSaveImg(fileName);

  return 0;
}

//-----------------------------------------------------------------------------

