/*
  Copyright (C) 2012  Torsten Brischalle

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/
*/

#include "lua_func_vecgfx.h"

#include "lauxlib.h"
#include "lstate.h"

#include "InterfaceGUI.h"

//-----------------------------------------------------------------------------

int lua_func_vecgfx_clear(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  iGui->VecGfxClear();

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_draw_line(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x1 = luaL_checknumber(lua, 1);
  double  y1 = luaL_checknumber(lua, 2);
  double  x2 = luaL_checknumber(lua, 3);
  double  y2 = luaL_checknumber(lua, 4);

  iGui->VecGfxDrawLine(x1, y1, x2, y2);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_draw_circle(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);
  double  r = luaL_checknumber(lua, 3);

  iGui->VecGfxDrawCircle(x, y, r);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_draw_ellipse(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);
  double  rx = luaL_checknumber(lua, 3);
  double  ry = luaL_checknumber(lua, 4);

  iGui->VecGfxDrawEllipse(x, y, rx, ry);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_draw_rect(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);
  double  w = luaL_checknumber(lua, 3);
  double  h = luaL_checknumber(lua, 4);

  iGui->VecGfxDrawRect(x, y, w, h);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_draw_polygon(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  QPolygonF polygon;

  luaL_checktype(lua, 1, LUA_TTABLE);

  for (lua_pushnil(lua); lua_next(lua, 1); lua_pop(lua, 1))
  {
    luaL_checktype(lua, -1, LUA_TTABLE);

    lua_rawgeti(lua, -1, 1);
    double x = luaL_checknumber(lua, -1);
    lua_pop(lua, 1);

    lua_rawgeti(lua, -1, 2);
    double y = luaL_checknumber(lua, -1);
    lua_pop(lua, 1);

    polygon << QPointF(x, y);
  }

  iGui->VecGfxDrawPolygon(polygon);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_fill_circle(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);
  double  r = luaL_checknumber(lua, 3);

  iGui->VecGfxFillCircle(x, y, r);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_fill_ellipse(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);
  double  rx = luaL_checknumber(lua, 3);
  double  ry = luaL_checknumber(lua, 4);

  iGui->VecGfxFillEllipse(x, y, rx, ry);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_fill_rect(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);
  double  w = luaL_checknumber(lua, 3);
  double  h = luaL_checknumber(lua, 4);

  iGui->VecGfxFillRect(x, y, w, h);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_fill_polygon(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  QPolygonF polygon;

  luaL_checktype(lua, 1, LUA_TTABLE);

  for (lua_pushnil(lua); lua_next(lua, 1); lua_pop(lua, 1))
  {
    luaL_checktype(lua, -1, LUA_TTABLE);

    lua_rawgeti(lua, -1, 1);
    double x = luaL_checknumber(lua, -1);
    lua_pop(lua, 1);

    lua_rawgeti(lua, -1, 2);
    double y = luaL_checknumber(lua, -1);
    lua_pop(lua, 1);

    polygon << QPointF(x, y);
  }

  iGui->VecGfxFillPolygon(polygon);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_draw_text(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  double  x = luaL_checknumber(lua, 1);
  double  y = luaL_checknumber(lua, 2);

  iGui->VecGfxDrawText(x, y, luaL_checkstring(lua, 3));

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_set_color(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  if (lua_isnumber(lua, 1))
  {
    int red = luaL_checkinteger(lua, 1);
    int green = luaL_checkinteger(lua, 2);
    int blue = luaL_checkinteger(lua, 3);

    iGui->VecGfxSetColor(red, green, blue);
  }
  else
  {
    iGui->VecGfxSetColor(luaL_checkstring(lua, 1));
  }

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_set_fill_color(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  if (lua_isnumber(lua, 1))
  {
    int red = luaL_checkinteger(lua, 1);
    int green = luaL_checkinteger(lua, 2);
    int blue = luaL_checkinteger(lua, 3);

    iGui->VecGfxSetFillColor(red, green, blue);
  }
  else
  {
    iGui->VecGfxSetFillColor(luaL_checkstring(lua, 1));
  }

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_save_img(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  QString fileName = luaL_checkstring(lua, 1);
  int     width = 1280;
  int     height = 1024;

  if (!lua_isnoneornil(lua, 2))
    width = luaL_checkinteger(lua, 2);

  if (!lua_isnoneornil(lua, 3))
    height = luaL_checkinteger(lua, 3);

  iGui->VecGfxSaveImg(fileName, width, height);

  return 0;
}

//-----------------------------------------------------------------------------

int lua_func_vecgfx_save_svg(lua_State *lua)
{
  InterfaceGUI *iGui = lua->iGui;

  QString fileName = luaL_checkstring(lua, 1);

  iGui->VecGfxSaveSvg(fileName);

  return 0;
}

//-----------------------------------------------------------------------------

